/*****************************************************************************
 * Can.c
 *
 * Authors:
 *
 *    Valter Filipe Silva
 *    vfs@estga.ua.pt
 *
 *****************************************************************************/

#include "can.h"

unsigned char * fillCanRegisters (Message msg, unsigned char buff) 
{
    unsigned char *ptr,i,*ptrAux;
	
	
	if (buff == BUFFER_0)
	{
		if (TXB0REQ) return (NULL);
		else ptr = &TXB0SIDH;
	}
	
	if (buff == BUFFER_1)
	{
		if (TXB1REQ) return (NULL);
		else ptr = &TXB1SIDH;
	}
	if (buff == BUFFER_2)
	{
		if (TXB2REQ) return (NULL);
		else ptr = &TXB2SIDH;
	}
	
	ptr--;
	ptrAux=ptr;
	
	ptr++;
	*ptr++ = ((msg.stdID>>3)&0xff);   
	*ptr++ = ((msg.stdID<<5)&0xE0); 
	ptr+=2;

	
	msg.len=(msg.len>8?8:msg.len);
	*ptr++ = msg.len;  // set DLC
    
	for (i=0;i<msg.len;i++)
		ptr[i] = msg.data[i];

	return (ptrAux);
}



void setPriority (unsigned char buff, unsigned char priority)
{
    unsigned char *ptr; 
	
	
	if (buff == BUFFER_0)
		ptr = (unsigned char*)(&TXB0CON);
	else if (buff == BUFFER_1)
		ptr = (unsigned char*)(&TXB1CON);
	else if (buff == BUFFER_2)
		ptr = (unsigned char*)(&TXB2CON);
	
	(*ptr) &= 0xF8;                       //Colocar os bits de prioridade a zero.
	(*ptr)=(*ptr)|(priority&0x03);   //Programar a prioridade da mensagem.
	
}



void incrementAllPriority ()
{
	
	TXB0CON = (TXB0CON&0xF8)|((TXB0CON+1)&0x03);
	TXB1CON = (TXB1CON&0xF8)|((TXB1CON+1)&0x03);
	TXB2CON = (TXB2CON&0xF8)|((TXB2CON+1)&0x03);
	
}


signed char sendCanMessage(Message msg) 
{
	
	unsigned char * ptrAux;
	unsigned char i;
	
	for (i=BUFFER_0; i<= BUFFER_2; i++)
        if ((ptrAux=fillCanRegisters (msg,i))!=NULL) break;
		
		if (ptrAux!=NULL)
		{
			(*ptrAux) =(*ptrAux) | 0x08;
			return (OK);	    
		}   
		else return (KO);
		
}

signed char sendCanMessageBuff (Message msg, unsigned char buff) 
{
	
	unsigned char * ptrAux;
	
	if ((ptrAux=fillCanRegisters (msg,buff))==NULL)
		return (KO);
	(*ptrAux) =(*ptrAux) | 0x08;
	return (OK);	    
}



void initCAN(unsigned char InitCanMode,unsigned char interrupts, unsigned char brgcon1,unsigned char brgcon2,unsigned char brgcon3)
{	
	//Colocar o Controlador em Configuration Mode.
	REQOP2=1;

	while( !OPMODE2 );      //Only when the status bit OPMODE2 has a high level,
	//can the initialization be performed.
	//Afterwards, the configuration registers, the
	//acceptance mask registers, and the acceptance filter
	//registers can be written.
	//The module is activated by setting the REQOP
	//control bits to zero.
	
	BRGCON1= brgcon1;
	
	BRGCON2= brgcon2;
	
	BRGCON3= brgcon3;
	
	RXM0SIDH=0;
	RXM0SIDL=0;
	RXM1SIDH=0;
	RXM1SIDL=0;
	RXF0SIDL=0;
	RXF1SIDL=0;
	
	InitCanMode =InitCanMode<< 5;
	CANCON = InitCanMode;
	while ((CANSTAT & 0xE0) != InitCanMode);
	
	
	//CAN Module I/O Control Register.
	ENDRHI = 1;     //CANTX pin will drive VDD when recessive.
	CANCAP = 1;     //Enable CAN capture.
    
	
	RXB0IE=(~interrupts) & 0x01;
	RXB1IE=(~interrupts)>>1 & 0x01;
    
	TXB0IE=(~interrupts)>>2 & 0x01;
	TXB1IE=(~interrupts)>>3 & 0x01;
	TXB2IE=(~interrupts)>>4 & 0x01;

	RXB0IF=0;
	RXB1IF=0;
	
	TXB0IF=0;
	TXB1IF=0;
	TXB2IF=0;

	RXB0FUL=0;
	RXB1FUL=0;
	RXB0DBEN=1;
	
	RXB0M1=0;
	RXB0M0=0;
    
	RXB1M1=0;		// receive all message (including messages with errors)
	RXB1M0=0;

	TRISB3=1;
	TRISB2=0;
}


void setMask(unsigned short SetMaskStdId, unsigned char SetMaskBuff)
{
	unsigned char SetMaskActMode, *SetMaskAddr;
	TRISB3=1;
	TRISB2=1;

	//Colocar o Controlador em Configuration Mode.
	SetMaskActMode = CANSTAT;
	
	REQOP2=1;
	while( !OPMODE2 );      //The mask and filter registers can only be modified
	//when the PIC18FXX8 is in Configuration mode. The
	//mask and filter registers cannot be read outside of Configuration
	//mode. When outside of Configuration mode,
	//all mask and filter registers will be read as 0.
	
	SetMaskAddr = &RXM0SIDH + (SetMaskBuff&0x01)*4;
	
	*SetMaskAddr++ = ( ( SetMaskStdId & 0x07F8 )>>3 );

	*SetMaskAddr = (  ( SetMaskStdId & 0x0007 )<<5 ) ; //| ( ( (SetMaskExtId) & 0x00030000 )>>16 ) );

	CANCON = SetMaskActMode;
	
	while( CANSTAT != SetMaskActMode );
	TRISB3=1;

	TRISB2=0;
}
void setFilter(unsigned short SetFilterStdId/*, unsigned long SetFilterExtId, unsigned char SetFilterExtIdBit*/, unsigned char SetFilterBuff)
{

	unsigned char SetFilterActMode, *SetFilterAddr;
	TRISB3=1;
	TRISB2=1;

	//Colocar o Controlador em Configuration Mode.
	SetFilterActMode = CANSTAT;

	REQOP2=1;
	
	while( !OPMODE2 );      //The mask and filter registers can only be modified
	//when the PIC18FXX8 is in Configuration mode. The
	//mask and filter registers cannot be read outside of Configuration
	//mode. When outside of Configuration mode,
	//all mask and filter registers will be read as 0.

	SetFilterAddr = &RXF0SIDH + SetFilterBuff*4;
	*SetFilterAddr++ = ( ( SetFilterStdId & 0x07F8 )>>3 );
	*SetFilterAddr = ( ( SetFilterStdId & 0x0007 )<<5 );
	*SetFilterAddr&=0xF7;    // Do not accept messages with extended identifiers
	CANCON = SetFilterActMode;
	
	while( CANSTAT != SetFilterActMode );
	TRISB3=1;
	TRISB2=0;
}


