/*****************************************************************************
 * Can.h
 *
 * Authors:
 *
 *    Valter Filipe Silva
 *    vfs@estga.ua.pt
 *
 *****************************************************************************/
#if !defined(_CAN_H_)

#define _CAN_H_


#include <pic18.h>
#include <string.h>
#include <stdio.h>
/***************************
General Defines
****************************/
#if !defined(OK)

#define OK 		0
#define KO 		-1

#endif

/*****************************
Struct of messages
******************************/

typedef struct 
{
	unsigned short stdID;
	unsigned char len;
	unsigned char data[8];
} Message;


/*****************************
CAN Operation Mode
******************************/


#define NORMAL_MODE 		0
#define DISABLE_MODE 		1
#define LOOPBACK_MODE 		2
#define LISTEN_ONLY_MODE 	3
#define CONFIGURATION_MODE 	4


/*****************************
CAN Priorities
******************************/



#define PRIORITY_LEVEL_3 	3 //Highest priority
#define PRIORITY_LEVEL_2 	2 
#define PRIORITY_LEVEL_1 	1 
#define PRIORITY_LEVEL_0 	0 //Lowest priority




/*********************************
CAN Transmitt buffer definitions
*********************************/



#define BUFFER_0 		0
#define BUFFER_1 		1 
#define BUFFER_2 		2 



/*****************************
CAN Type of ID
******************************/

#define STD_ID 			0
#define EXT_ID 			1

/**********************************
Mask for initCAN
***********************************/

#define RXB0_INTERRUPT		0xFE
#define RXB1_INTERRUPT		0xFD
#define TXB0_INTERRUPT		0xFB
#define TXB1_INTERRUPT		0xF7
#define TXB2_INTERRUPT		0xEF
#define NO_INTERRUPT		0xFF

//#define CAN_BAUD 		250      // CAN Baud Rate in Kbps


#if XTAL==40


#if CAN_BAUD==1000
#endif

#if CAN_BAUD==500
#endif

#if CAN_BAUD==250
#define INI_CAN_PARAMETERS 0x47,0x91,0x03     // REVER OS RESTANTES VALORES 
#endif

#if CAN_BAUD==125
#define INI_CAN_PARAMETERS 0x4F,0x91,0x03     // REVER OS RESTANTES VALORES 
#endif
#endif

#if XTAL==20

#if CAN_BAUD==1000
#endif

#if CAN_BAUD==500
#endif

#if CAN_BAUD==250
#endif

#if CAN_BAUD==125
#define INI_CAN_PARAMETERS 0x47,0x91,0x03     // *** REVER OS VALORES 
#endif
#endif

#if XTAL==10

#if CAN_BAUD==1000
#endif

#if CAN_BAUD==500
#endif

#if CAN_BAUD==250
#endif

#if CAN_BAUD==125
#endif
#endif

/******************************
Functions prototypes
*******************************/

void initCAN(unsigned char,unsigned char, unsigned char,unsigned char,unsigned char);
void setMask(unsigned short, unsigned char);
void setFilter(unsigned short, unsigned char);
signed char sendCanMessage(Message);
signed char sendCanMessageBuff(Message, unsigned char);
unsigned char *fillCanRegisters(Message, unsigned char);
void setPriority (unsigned char, unsigned char);
void incrementAllPriority ();

#endif
