/*****************************************************************************
 * Compiled in PIC18F258-20MHz-PLLOn
 * Compiled with PICC18#v8.30 with
 *    Global optimizations on = 9
 *    Assembler optimization on
 *    Double size = 4 bytes
 *    Fast double routines
 *
 *    Use project: rcaisnd.prj
 *
 * Author:
 *    Paulo Pedreiras
 *    
 *****************************************************************************/
#include <pic18.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

/* CAN stuff */
#define XTAL 20
#define CAN_BAUD 		125      // CAN Baud Rate in Kbps
#include "can.h"

#define FALSE 0
#define TRUE !FALSE

unsigned int ticks;	// Global var. for setting Timer 0 interval
unsigned char T0_fired;


int id;                              ///< Received identifier of a CAN message
unsigned char len;		     ///< lenght of the received message in CAN
unsigned char mess[8];		    ///< The received message
unsigned char CAN_rxflag;

#define GID 0x3 /* Group ID, {1,2,...,8} */

/* Function prototypes */
void putch(unsigned char);
void SetTimer0(unsigned int );
void interrupt isr();

void main(void)
{
	Message cmsg; /* Can message */
	int i;
				
	/* Init CAN layer */
	initCAN(NORMAL_MODE, RXB0_INTERRUPT & RXB1_INTERRUPT & TXB0_INTERRUPT & TXB1_INTERRUPT & TXB2_INTERRUPT, INI_CAN_PARAMETERS);
	setMask( 0b11100000000, 0);		// Mask for RX buffer 0
	setFilter( 0b11100000000, 0);		// Filter 0,1 -> RX buffer 0 (Filter 2-5 Rx buffer 1)
	setMask( 0b11100000000, 1);		// Mask for RX buffer 0
	setFilter( 0b11100000000, 2);		// Filter 0,1 -> RX buffer 0 (Filter 2-5 Rx buffer 1)
	
	CAN_rxflag=FALSE;

	T0_fired=FALSE;
	SetTimer0(10); // Timer 0 fires every 5ms			
	
	GIE = 1; // Enable ints
	
	ADCON1=0x7; // PORTA pins for digital I/O
	TRISA=0x00; // PORTA pins as outputs;
	PORTA = 0x55;
	for(;;) 
	{
		if(T0_fired)  // Periodic timer expired
		{
			/* Send CAN message */
			cmsg.len=8 ; // Message with 8 data bytes
			cmsg.stdID=0x700 | GID; // CAN ID (*NOTE* 11 bis ); 
			for(i=0;i<7;i++)	
				cmsg.data[i]=i*10+GID;
		
			sendCanMessage (cmsg);
		
			T0_fired = FALSE;
			PORTA= ~PORTA;
		}
		
		if(CAN_rxflag) // Message receivedp
		{
			printf("\n\rID=%x [%x:%x]",id,mess[0],mess[1]);
			CAN_rxflag=FALSE;
		}
	}

	/* Code never reached */
	return ;
}


/* Function implementation */
void putch(unsigned char byte) {
	/* output one byte */
	while(TRMT==0) continue;	/* set when register is empty */
	TXREG = byte;
}

void SetTimer0(unsigned int interval)
{	/* Interval in ms */
	
	T0CON=0x00; // Stop clock

	T08BIT=0; // 16 bit timer
	T0CS=0;	  // Internal clock
	PSA=1;    // No prescaler
		
	ticks = 0xFFFF-interval*1000*5; // Xtal = 20MHz, no pre-scaling (tick period = 200ns)
	WRITETIMER0(ticks);
	
	TMR0IF=0;
	TMR0IE=1;                  // Timer 0 interrupt enable
	TMR0ON=1;		   // Timer 0 On
}



/* Interrupt routime 
Just one in PIC for all interrupts sources
*/
void interrupt isr()
{
	static int i;	
	static unsigned char *ptr;
	
	if (TMR0IF && TMR0IE)
	{
		TMR0IF=0;    		// The flag must be reset by software
		WRITETIMER0(ticks);	// Set Tmr 0 interval counter
		T0_fired = TRUE;
	}
	
	if ( (TXB0IF && TXB0IE) || (TXB1IF && TXB1IE) || (TXB2IF && TXB2IE) )
	{
		if (TXB0IF)
			TXB0IF=0;
		else 
		{	if (TXB1IF)
				TXB1IF=0;
			else 
				TXB2IF=0;
		}
	}
		
	if ( (RXB0IF && RXB0IE) || (RXB1IF && RXB1IE) ) // Message received
	{	
		if (RXB0FUL)  			
		{
			RXB0IF=0;  ///////
			RXB0FUL=0;
			ptr=(unsigned char *)(&(RXB0SIDH));
		}
		else if (RXB1FUL)
		{
			RXB1IF=0;
			RXB1FUL=0;
			ptr=(unsigned char *)(&(RXB1SIDH));			
		}
		
		/* Get CAN message contents */
		id = (*ptr++) << 3; 
		id |= ( (*ptr++) >>5 ) & 0x07;
		ptr += 3;	
		len = ( *(ptr-1) & 0x0F ); 
		for ( i=0; i<len; i++ )   	// copy of can message to buffers !!!!
			mess[i] = ptr[i];
		
		CAN_rxflag = TRUE;
		
	}

}
